﻿package com.core {
	
	/*
	All graphics, design, and additional programming by: Anthony Scavarelli - http://blog.anthony-scavarelli.com/
	Special thanks to:
	- Fuel Industries for the time to build this. http://www.fuelindustries.com/
	- The Papervision team for their wonderful Papervision classes
	- The Kingnare css and swf control styles by Jin Xin here: http://www.scalenine.com/themes/kingnare/Kingnare.html#
	- Pablo Bandin (http://pablobandin.wordpress.com), for the basis of an animation control class
	- and the many Papervision and Flex resources on teh interwebs for help with the set-up and many problems encountered building this app.
	
	This application is available for any use, or modeification. I would appreciate any feedback or comments on how you have been able to use it or modify it at it’s original source:
	*/
	
	import com.objects.DAEMC;
	import com.objects.PaperBase;
	
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.ui.Keyboard;
	
	import org.papervision3d.core.math.Number3D;
	import org.papervision3d.events.FileLoadEvent;
	import org.papervision3d.materials.BitmapFileMaterial;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.materials.special.CompositeMaterial;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.objects.primitives.Cube;
	
	public class ColladaViewer extends PaperBase {
		
		public const AXIS_LENGTH:Number = 300;
		public const AXIS_WIDTH:Number = 3;
		
		//events
		public static const NEW_LOAD:String = "newLoad";
		public static const MODEL_LOADED:String = "modelLoaded";
		public static const ANIM_LOADED:String = "animationsLoaded";
		public static const FRAME_CHANGE:String = "framesChanged";
		public static const ANIM_CHECK:String = "aCheck";
		public static const ANIM_COMP:String = "aComplete";
		public static const LOAD_ERROR:String = "LoadErr";
		
		protected var _model:DAEMC = null; //animated model
		//protected var _registCon:DisplayObject3D = new DisplayObject3D(); //embpty container to put model into to change registration pt
		protected var _axis:DisplayObject3D = new DisplayObject3D(); //will hold line axis
		protected var _xAxis:Cube;
		protected var _yAxis:Cube;
		protected var _zAxis:Cube;
		
		//holding references to "vectors" on up and side axis of target so I know where to pan camera
		protected var _targetUpAxis:Number3D;
		protected var _targetSideAxis:Number3D;
		
		protected var _dumTarget:Cube; //target for camera
		protected var _dumTargetCol:ColorMaterial; //need a mat to create cube
		
		protected var _sceneWidth:Number;
        protected var _sceneHeight:Number;
        protected var _cameraYaw:Number = 0;
        protected var _cameraPitch:Number = 0;
        protected var _cameraZoom:Number = 0;
        protected var _yCamPos:Number = 0;
        protected var _xCamPos:Number = 0;
        
        protected var _camRotating:Boolean = false;
        protected var _camZooming:Boolean = false;
        protected var _camPanning:Boolean = false;
        
    	protected var _altPressed:Boolean = false;
        
        protected var _mFileName:String = ""; //model filePath
        protected var _mTextureName:String = ""; //texture filePath
        
        protected var _isMouseOver:Boolean = false;
        protected var _onThisFrame:int = 0;
        protected var _stepFrame:Boolean = true; //checks to see if animation should be stopped on next frame
        protected var _modelAnimated:Boolean = false;
        protected var _animLoading:Boolean = false;
        
        protected var _modelColour:int = 0xe19620;
        protected var _wireColour:int = 0x333333;
        
        //materials
		protected var _wireMat:WireframeMaterial;
		protected var _bmMat:BitmapFileMaterial; 
		protected var _compositeMat:CompositeMaterial;
		protected var _colorMat:ColorMaterial;
		
		protected var _autoTexture:Boolean = false;
		
		
		/****************************************************************/
		/*********************** initialization *************************/
		/****************************************************************/
		
		public function ColladaViewer():void {	
			//default parameters
		 	this.opaqueBackground = "0x000000";
            this.addEventListener(Event.ADDED_TO_STAGE, addedToStage);
		}
		
		protected function addedToStage(e:Event):void {
			this.removeEventListener(Event.ADDED_TO_STAGE, addedToStage);
			
			//need this as any parameters modified within mxml do not come into effect until after constructor called
			//and stage references can only be made after constructor called
			_sceneWidth = this.width;
            _sceneHeight = this.width;
            
            this.enabled=true;
            this.mouseEnabled = false;
            this.addEventListener(MouseEvent.ROLL_OVER, mouseOver);
			this.addEventListener(MouseEvent.ROLL_OUT, mouseOver);
			this.stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown);
			
			init(_sceneWidth,_sceneHeight);
			
			//create the three axis
			
			//create matsList
			//x - red
			var __xMatsList:MaterialsList = new MaterialsList({all:new ColorMaterial(0xFF0000)});
			_xAxis = new Cube(__xMatsList, AXIS_LENGTH, AXIS_WIDTH, AXIS_WIDTH);
			
			//y - green
			var __yMatsList:MaterialsList = new MaterialsList({all:new ColorMaterial(0x00FF00)});
			_yAxis = new Cube(__yMatsList, AXIS_WIDTH, AXIS_LENGTH, AXIS_WIDTH);
			
			//z - blue
			var __zMatsList:MaterialsList = new MaterialsList({all:new ColorMaterial(0x0000FF)});
			_zAxis = new Cube(__zMatsList, AXIS_WIDTH, AXIS_WIDTH ,AXIS_LENGTH);
			
			//adding to another container
			_axis.addChild(_xAxis);
			_axis.addChild(_yAxis);
			_axis.addChild(_zAxis);
			
			//arranging axis so that they are connected at ends
			_xAxis.x += AXIS_LENGTH/2;
			_yAxis.z += AXIS_LENGTH/2;
			_zAxis.y += AXIS_LENGTH/2;
			
			default_scene.addChild(_axis);
			//default_camera.lookAt(_axis);
			
			createCameraTarget();
			default_camera.fov = 50;
		}
		
		/****************************************************************/
		/******************* end initialization *************************/
		/****************************************************************/
		
		
		/****************************************************************/
		/*********************** events *********************************/
		/****************************************************************/
		
		protected function animationComplete(e:Event):void {
        	trace("ANIMATIONS COMPLETE");
        	
        	_animLoading = false;
        	dispatchEvent(new Event(ColladaViewer.ANIM_COMP));
        }
        
        protected function checkForAnim(e:Event):void {
        	_model.removeEventListener(DAEMC.CHECK_FOR_ANIM, checkForAnim);
        	
        	if (_model != null) {
        		_modelAnimated = _model.isAnimated;
        		_animLoading = _model.isAnimated;
        		dispatchEvent(new Event(ColladaViewer.ANIM_CHECK));
        		trace("MODEL ANIMATED?: " + _modelAnimated);
        	}
        }
		
		protected function getAnimationData(e:FileLoadEvent):void {
			_model.removeEventListener(FileLoadEvent.ANIMATIONS_COMPLETE, getAnimationData);
			this.dispatchEvent(new Event(ColladaViewer.ANIM_LOADED));
		}
		
		protected function keyDown(e:KeyboardEvent):void {
			this.stage.removeEventListener(KeyboardEvent.KEY_DOWN, keyDown);
			this.stage.addEventListener(KeyboardEvent.KEY_UP, keyUp);
			
			if (e.keyCode == Keyboard.ALTERNATE) {
				_altPressed = true;
			}
		}
		
		protected function keyUp(e:KeyboardEvent):void {
			this.stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown);
			this.stage.removeEventListener(KeyboardEvent.KEY_UP, keyUp);
			
			if (e.keyCode == Keyboard.ALTERNATE) {
				_altPressed = false;
			}
		}
		
		protected function modelLoaded(e:Event):void {
			loopAnimation();
			this.removeEventListener(FileLoadEvent.LOAD_COMPLETE, modelLoaded);
			this.dispatchEvent(new Event(ColladaViewer.MODEL_LOADED));
		}
		
		protected function mouseOver(e:Event):void {
			switch (e) {
				case MouseEvent.ROLL_OVER: _isMouseOver=true; break;
				case MouseEvent.ROLL_OUT: _isMouseOver=false; break;
			}
		}
		
		/****************************************************************/
		/*********************** end events *****************************/
		/****************************************************************/
		
		
		/****************************************************************/
		/*********************** methods ********************************/
		/****************************************************************/
		
		public function cameraPitch(__pitch:Number):void {
			_cameraPitch += __pitch;
		}
		
		public function cameraYaw(__yaw:Number):void {
			_cameraYaw += __yaw;
		}
		
		public function cameraZoom(__zoom:Number):void {
			_cameraZoom = __zoom;
		}
		
		protected function createCameraTarget():void {
			//create dummy cube as target camera now
             _dumTargetCol = new ColorMaterial(0xFF0000);
            _dumTarget = new Cube(new MaterialsList({all:_dumTargetCol}),10,10,10); //colour front face
            _dumTarget.scale = 3;
            _dumTarget.pitch(-40);
            _dumTarget.x = 0;
            _dumTarget.y = 0;
            _dumTarget.z = 0;
            default_scene.addChild(_dumTarget);
            default_camera.lookAt(_dumTarget);
            _dumTarget.lookAt(default_camera);
            //_dumTargetCol.invisible = true; //don't want to see the box
            
            //now to find faces that have appropriate normals I can track for camera movement purposes later
            for (var i:int = (_dumTarget.geometry.faces.length - 1); i >= 0; --i) {
            	//trace("****" + _dumTarget.geometry.faces[i].faceNormal);
            	var __t:Number3D = _dumTarget.geometry.faces[i].faceNormal;
            	
            	//setting top axis
            	if  ((__t.x == 0) && (__t.y == 1) && (__t.z == 0)) {
            		_targetUpAxis = __t;
            	}
            	
            	//setting side axis
            	if  ((__t.x == 1) && (__t.y == 0) && (__t.z == 0)) {
            		_targetSideAxis = __t;
            	}
            }
		}
		
		//deinitialize 3D stuff
        public function deInit3D():void {
        	
        	
        	default_scene.removeChild(_model);
        	_model = null;
        	_mFileName = "";
        	_mTextureName = "";
        	_modelAnimated = false;
        	_animLoading = false;
        	_stepFrame = true;
        }
		
		public override function init3d():void {
			
			if (_mFileName != "") {
				
				_model = new DAEMC(false, null, 0);
				resetCamera();

				if (_mTextureName != "") {
					_bmMat = new BitmapFileMaterial(_mTextureName, true);
					_wireMat = new WireframeMaterial(_wireColour);
					_colorMat = new ColorMaterial(_modelColour);
					_compositeMat = new CompositeMaterial();
					
					_wireMat.invisible = true;
					_colorMat.invisible = true;
					
					_compositeMat.addMaterial(_bmMat);
					_compositeMat.addMaterial(_colorMat);
					_compositeMat.addMaterial(_wireMat);

					_model.addEventListener("Loaded", modelLoaded);
					_model.addEventListener(FileLoadEvent.ANIMATIONS_COMPLETE, getAnimationData);
					_model.addEventListener(FileLoadEvent.LOAD_COMPLETE, checkForAnim);
					_model.addEventListener(DAEMC.ANIM_COMPLETE, animationComplete);
					
					 try {
						if (_autoTexture) {
							_model.load(_mFileName);
						} else {
							_model.load(_mFileName,  new MaterialsList ( { all:_compositeMat } ));
						}
					} catch (err:Error) {
						//now to clean-up so new models can loaded
						dispatchEvent(new Event(ColladaViewer.LOAD_ERROR)); //dispath error so load can be reset
					}
					
					if (_model != null) {
						default_camera.lookAt(_model);
					} 
				
				} else {
					
					_colorMat = new ColorMaterial(_modelColour);
					_wireMat = new WireframeMaterial(_wireColour);
					_compositeMat = new CompositeMaterial();
					
					_compositeMat.addMaterial(_colorMat);
					_compositeMat.addMaterial(_wireMat);

					_model.addEventListener("Loaded", modelLoaded);
					_model.addEventListener(FileLoadEvent.ANIMATIONS_COMPLETE, getAnimationData);
					_model.addEventListener(FileLoadEvent.LOAD_COMPLETE, checkForAnim);
					_model.addEventListener(DAEMC.ANIM_COMPLETE, animationComplete);

					 try {
						if (_autoTexture) {
							_model.load(_mFileName);
						} else {
							_model.load(_mFileName,  new MaterialsList ( { all:_compositeMat } ));
						}
					} catch (err:Error) {
						//now to clean-up so new models can loaded
						deInit3D();
					}
					if (_model != null) {
						default_camera.lookAt(_model);
					} 
				}

				if (_model != null) {
					
					default_scene.addChild(_model);
				}
			}	
        }
        
        protected function loopAnimation():void {
			//setting this to model so there will be an error here as "this" does not have this function
			if (_model != null) {
				_model.makeLooping();
			}
		}
		
		public function moveCamera(__xMov:Number = 0, __yMov:Number = 0):void {
			_xCamPos = __xMov;
			_yCamPos = __yMov;
		}

		public function pauseAnim():void {
			if (_model != null) {
				_model.stop();
			}
		}

		public function playAnim():void {
			if (_model != null) {
				_model.play();
			}
		}
        
        //enterFrame function
		protected override function processFrame():void {
			//enter any additional animation logic here
			
			//pitch and yaw
			if (camRotating && !camPanning) {
				default_camera.orbit(_cameraPitch,_cameraYaw,true,_dumTarget); //rotate camera around scene but looking towards dumTarget
				_dumTarget.lookAt(default_camera); //keep object rotating and looking to camera
				//trace("*" + _targetUpAxis + " " + _targetSideAxis);
			}
			
			//zoom
			if (camZooming) {
				if (_cameraZoom < 0) {
					default_camera.moveBackward(Math.abs(_cameraZoom));
					_cameraZoom= 0;
				} else {
					default_camera.moveForward(Math.abs(_cameraZoom));
					_cameraZoom= 0;
				}
				
				this.camZooming = false; //reset as mouseWheel change will set to true again
			}
			
			//x-y movement
			if (camPanning && !camRotating) {
				
				//moveCamera and target
				if (_xCamPos > 0) {
					default_camera.moveRight(Math.abs(_xCamPos));
					_dumTarget.moveRight(Math.abs(_xCamPos))
					_xCamPos = 0;
				} else {
					default_camera.moveLeft(Math.abs(_xCamPos));
					_dumTarget.moveLeft(Math.abs(_xCamPos))
					_xCamPos = 0;
				}
				
				if (_yCamPos > 0) {
					default_camera.moveUp(Math.abs(_yCamPos));
					_dumTarget.moveUp(Math.abs(_yCamPos))
					_yCamPos = 0;
				} else {
					default_camera.moveDown(Math.abs(_yCamPos));
					_dumTarget.moveDown(Math.abs(_yCamPos))
					_yCamPos = 0;
				}
				
				//then look at target
				default_camera.lookAt(_dumTarget);
				_dumTarget.lookAt(default_camera);
			}
			
			//check for frame change to update animation slider
			if (_model != null) {
				if (_model.currentFrame != _onThisFrame) {
					dispatchEvent(new Event(ColladaViewer.FRAME_CHANGE));
					_onThisFrame = _model.currentFrame;
				}
			}
		}
        
        //to release any references to model so I can reload when the dae xml is saved
        public function releaseModel():void {
        	trace("releasing Model");
        	default_scene.removeChild(_model);
        	_model = null;
        	_modelAnimated = false;
        	_animLoading = false;
        	_stepFrame = true;
        }
		
		public function resetCamera():void {
			
			default_camera.x = 0;
			default_camera.y = 0;
			default_camera.z = -1000;
			
			_dumTarget.x = 0;
			_dumTarget.y = 0;
			_dumTarget.z = 0;
			
			default_camera.lookAt(_dumTarget);
			_dumTarget.lookAt(default_camera);
		}
		
		public function set scrubFrame(__gotoFrame:int):void {
			
			if (_model != null) {
				//_model.addEventListener(DAEMC.FRAME_EXECUTED, stopThisFrame);
					
				_model.gotoAndPlay(__gotoFrame);
				_model.stoppaThis = true;
				//_stepFrame = true;
			}
		}
		
		public function stopAnim():void {
			if (_model != null) {
				_model.gotoAndStop(1);
			}
		}
		
		/****************************************************************/
		/*********************** end methods ****************************/
		/****************************************************************/
		
		/****************************************************************/
		/*********************** properties *****************************/
		/****************************************************************/
		
		public function get altDown():Boolean {
			return _altPressed;
		}
		
		public function get animLoading():Boolean {
			return _animLoading;
		}
		
		public function get autoTexture():Boolean {
			return _autoTexture;
		}
		
		public function set autoTexture(__bool:Boolean):void {
			_autoTexture = __bool;
		}
		
		public function get camPanning():Boolean {
			return _camPanning;
		}
		
		public function set camPanning(__newVal:Boolean):void {
			_camPanning = __newVal;
		}
		
		public function get camRotating():Boolean {
			return _camRotating;
		}

		public function set camRotating(__newVal:Boolean):void {
			_camRotating = __newVal;
		}
		
		public function get camZooming():Boolean {
			return _camZooming;
		}
		
		public function set camZooming(__newVal:Boolean):void {
			_camZooming = __newVal;
		}
		
		public function get currentFrame():int {
			if (_model != null) {
				return _model.currentFrame;
			} else {
				return 1;
			}
		}
		
		public function get currScale():Number {
			if (_model != null) {
				return _model.scale;
			} else return 0;
		}
		
		public function set currScale(newScale:Number):void{
			if (_model != null) {
				_model.scale = newScale;
			}
		}
		
		public function get invisAxis():Boolean {
			return _axis.visible;
		}
		
		public function set invisAxis(__invis:Boolean):void {
			_axis.visible = __invis;
		}
		
		public function get invisColor():Boolean {
			if (_colorMat != null) {
				return _colorMat.invisible;
			} return false;
		}
		
		public function set invisColor(__invis:Boolean):void {
			if (_colorMat != null) {
				_colorMat.invisible = __invis;
			}
		}
		
		public function get invisTarget():Boolean {
			return _dumTarget.visible;
		}
		
		public function set invisTarget(__invis:Boolean):void {
			_dumTarget.visible = __invis;
		}
		
		public function get invisWire():Boolean {
			if (_wireMat != null) {
				return _wireMat.invisible;
			} return false;
		}
		
		public function set invisWire(__invis:Boolean):void {
			if (_wireMat != null) {
				_wireMat.invisible = __invis;
			}
		}
		
		public function get isMouseOver():Boolean {
			return _isMouseOver;
		}
		
		public function get modColour():int{
			if (_colorMat != null) {
				return _colorMat.fillColor;
			} else return _modelColour;
		}
		
		public function set modColour(__newCol:int):void {
			if (_colorMat != null) {
				_colorMat.fillColor = __newCol;
			}
		}
		
		public function get modelAnimated():Boolean {
			return _modelAnimated;
		}
		
		public function get modelDir():String {
			return _mFileName;
		}
		
		public function set modelDir(newMod:String):void {
			_mFileName = newMod;
		}
		
		public function get modRotX():Number {
			if (_model != null) {
				return _model.rotationX;
			} else return 0;
		}
		
		public function set modRotX(newRot:Number):void {
			if (_model != null) {
				_model.rotationX = newRot;
			}
		}
		
		public function get modRotY():Number {
			if (_model != null) {
				return _model.rotationY;
			} else return 0;
		}
		
		public function set modRotY(newRot:Number):void {
			if (_model != null) {
				_model.rotationY = newRot;
			}
		}
		
		public function get modRotZ():Number {
			if (_model != null) {
				return _model.rotationZ;
			} else return 0;
		}
		
		public function set modRotZ(newRot:Number):void {
			if (_model != null) {
				_model.rotationZ = newRot;
			}
		}
		
		public function set newModCol(__newCol:int):void {
			_modelColour = __newCol;
		}
		
		public function set newWFCol(__newCol:int):void {
			_wireColour = __newCol;
		}
		
		public function get numTriangles():int {
			if (_model != null) {
				return _model.numFaces;
			} else {
				return NaN;
			}
		}
		
		public function get numVertices():int {
			if (_model != null) {
				return _model.numVertices;
			} else {
				return NaN;
			}
		}
		
		public function get numVisSceneObjects():int {
			if (_model != null) {
				return _model.numVisSceneObjects;
			} else {
				return NaN;
			}
		}
		
		public function get textureDir():String {
			return _mTextureName;
		}
		
		public function set textureDir(newTex:String):void {
			_mTextureName = newTex;
		}
		
		public function set thisFrame(__gotoFrame:int):void {
			
			if (_model != null) {
				
				if (__gotoFrame == (_model.currentFrame + 1)) {
					
					_model.play();
					_model.stoppaThis = true;

				} else if (__gotoFrame == (_model.currentFrame - 1)) {
					
					if ((_model.currentFrame - 1) > 1) {
						_model.gotoAndStop(__gotoFrame - 1);
					} else {
						_model.gotoAndStop(_model.totalFrames - 2);
					}
					
					_model.play();
					_model.stoppaThis = true;
				}
			}
		}
		
		public function get totalFrames():int {
			if (_model != null) {
				return _model.totalFrames;
			} else {
				return 50;
			}
		}
		
		public function get wfColour():int{
			if (_wireMat != null) {
				return _wireMat.lineColor;
			} else return _wireColour;
		}
		
		public function set wfColour(__newCol:int):void {
			if (_wireMat != null) {
				_wireMat.lineColor = __newCol;
			}
		}
		
		public function get WfThickness():Number {
			if (_wireMat != null) {
				return _wireMat.lineThickness;
			} else return 0;
		}
		
		public function set WfThickness(__newThick:Number):void {
			if (_wireMat != null) {
				_wireMat.lineThickness = __newThick;
			}
		}
		
		/****************************************************************/
		/*********************** end properties *************************/
		/****************************************************************/
	}
}